use <../FunctionalOpenSCAD/functional.scad>
use <../FunctionalOpenSCAD/double_fillet.scad>
use <../FunctionalOpenSCAD/webbing.scad>
use <../Libraries/bearing.scad>

$fa = 0.1;
$fs = 0.3;

bearingID = 8;
bearingOD = 22;
bearingTh = 7;
bearingTxt = "608-2RS";
// minimum thickness of base under center of bearing
bearingHeight = 5; 
// Fudge factor depending on 3D printing tolerances.  We want the bearing to press fit tight, but not too tight that it may crack the part (susceptibility to cracking also depends on surround thickness).
bearingClearance = 0.15;

// Min thickness of pillow blowck at top of bearing
bearingSurroundTh = 3;
// Radius of the bottom of the pillow block's profile curve
blockProfileBottomRadius = 5;
// Offset between the top and bottom radius of the pillow block profile, creating a slope.  use 0 for straight sides
radiusOffset = 5;
ribTh = 1;

// Mounting Bolt Diameter
boltD = 5;
// Distance from center-to-center for mounting bolts
boltSpacing = 40;
// Thickness of base under bolt heads
boltHeadHeight = 5; 
// Bolt head counterbore
boltHeadCounterbore = 10; 

slotClearance = 0.4; // added to bolt diameter for loose fit
slotWidth = 2; // center-to-center distance for mounting slots

showBearing = false;


//rotate([90,0,0]) 

pillow_block(bearingID, bearingOD, bearingTh, bearingTxt, bearingHeight, 
  bearingClearance, bearingSurroundTh, blockProfileBottomRadius, radiusOffset, ribTh, 
  boltD, boltSpacing, boltHeadHeight, boltHeadCounterbore, slotClearance, slotWidth, showBearing);

module slot(d, w) {
  if (w > 0) 
    hull() for(i=[-1,1]) translate([i*w/2,0]) circle(d=d);
  else
    circle(d=d);
}

// TODO support slotWidth 
module vertical_profile_centered() {
  x = boltSpacing-slotWidth;
  d1 = boltD + slotClearance;
  d2 = boltHeadCounterbore + ribTh*2;
  points = webbing(d2, d2, [x, 0], bearingTh);
  difference() {
    // primary vertical profile shape
    if (slotWidth > 0) { 
      union() {
        translate([-x/2,0]) poly2d(points);
        for(i=[0,1]) mirror([i,0]) translate([boltSpacing/2,0])
          slot(d2, slotWidth);
      }
    } else {
      translate([-x/2,0]) poly2d(points);
    }

    // bolt holes/slots
    for(i=[-1,1]) translate([i*boltSpacing/2,0])
      slot(d1, slotWidth);
  }
}

module vertical_profile_flat(rimTh) {
  x = boltSpacing-slotWidth;
  d1 = boltD + slotClearance;
  d2 = boltHeadCounterbore + ribTh*2;
  diff = (d2-bearingTh)/2;
  points = webbing(d2, d2, [x, 0], bearingTh-diff*2+rimTh*2);
  difference() {
    // primary vertical profile shape
    if (slotWidth > 0) {
      union() {
        translate([-x/2,0]) poly2d(points);
        for(i=[0,1]) mirror([i,0]) translate([boltSpacing/2,0])
          slot(d2, slotWidth);
        translate([-x/2,-bearingTh/2-diff]) square([x,bearingTh/2+diff]);
      }
    } else {
      translate([-x/2,0]) poly2d(points);
    }

    // bolt holes/slots
    for(i=[-1,1]) translate([i*boltSpacing/2,0])
      slot(d1, slotWidth);
  }
}


// return a list consisting of all element of "a" in order, then in reverse order, but mirrored about "axis"
// like taking a path and extending it with its mirror image
function ping_pong(a, axis=[1,0], end) = concat(a,reverse(mirror(axis, poly=a)), end==undef?[]:[end]); 

module pillow_block(bearingID, bearingOD, bearingTh, bearingTxt, bearingHeight, 
  bearingClearance, bearingSurroundTh, blockProfileBottomRadius, radiusOffset, ribTh, 
  boltD, boltSpacing, boltHeadHeight, boltHeadCounterbore, slotClearance, slotWidth, showBearing=false) {

  echo(str("pillow_block(",
    bearingID, ",", bearingOD, ",", bearingTh, ",\"", bearingTxt, "\",", bearingHeight, ",", 
    bearingClearance, ",", bearingSurroundTh, ",", blockProfileBottomRadius, ",", radiusOffset, ",", ribTh, ",", 
    boltD, ",", boltSpacing, ",", boltHeadHeight, ",", boltHeadCounterbore, ",", slotClearance, ",", slotWidth, ",", showBearing,
  ");")); 

  epsilon = 0.01;

  rimTh = 0.5 * (bearingOD - bearingID) / 4;
  rimID = bearingOD - rimTh*2;

  r1 = bearingOD/2+bearingSurroundTh;
  r2 = blockProfileBottomRadius;

  x1 = r1 + r2 + radiusOffset;
  x2 = boltSpacing/2+boltHeadCounterbore/2+slotWidth/2+ribTh;

  shaftHeight = bearingOD/2 + bearingHeight;
  totalHeight = bearingOD+bearingHeight+bearingSurroundTh;
  totalWidth = 2*max(x1,x2);
  totalDepth = max(bearingTh, boltHeadCounterbore+2*ribTh);

  d2 = boltHeadCounterbore + ribTh*2;
  diff = (d2-bearingTh)/2;
  
  echo(rimTh, rimID);
  echo(str("shaftHeight: ", shaftHeight, " totalHeight: ", totalHeight, " totalDepth: ", totalDepth));

  points = concat(
    translate([0,boltHeadHeight], 
      poly=double_fillet(
        h=totalHeight-boltHeadHeight, r1=r1, r2=r2, xoffset1=0, xoffset2=radiusOffset, closed=false
      )
    ),
    (x2 > x1) ? [[x2, boltHeadHeight], [x2, 0]] : [[x1, 0]]
  );

  //color([0,0,1])
  translate([0,ribTh+boltHeadCounterbore/2,0]) { // align flat face with x axis 

    difference() {
      intersection() {
        rotate([90,0]) translate([0,0,-totalDepth/2-1]) 
          linear_extrude(height=totalDepth+2, convexity=3)
            difference() {
              poly2d(ping_pong(points));
              translate([0,bearingHeight+bearingOD/2]) circle(d=rimID); 
              //echo(ping_pong(points));
            }
        translate([0,0,-1])
          linear_extrude(height=totalHeight+2, convexity=3)
            vertical_profile_flat(rimTh);
      }

      // bolt head counterbores
      translate([0,0,boltHeadHeight]) linear_extrude(height=totalHeight, convexity=2) for(i=[-1,1]) translate([i*boltSpacing/2,0])
        slot(boltHeadCounterbore+epsilon, slotWidth);

      translate([0,0]) rotate([-90,0,0]) 
        translate([0,-shaftHeight,-3*bearingTh/2-diff]) cylinder(d=bearingOD+bearingClearance, h=bearingTh*2);
    }

    if (showBearing)
      rotate([90,0]) translate([0,bearingOD/2+bearingHeight,-bearingTh/2+diff]) 
        bearing(bearingID, bearingOD, bearingTh, bearingTxt);
  }
}